function [P, X, C, lambda] = L1_ARG(Y, nb, rm, th, nss, ma, pdratio)
% New Version
% Cost Function : || Y - PX ||_1
% Y      : Observation Matrix
% P, X   : Basis and Coefficient Matrices
% nb     : Number of basis (rank)
% th     : Stopping Criterion
% rm     : Remove median value
% nss    : Number of step size (weighted median method)

if nargin < 3 || isempty(rm)
    rm = false;
end
if nargin < 4 || isempty(th)
    th = 1e-6;
end
if nargin < 5 || isempty(nss)
    nss = [];
end


[nd, ns] = size(Y);

if rm
    M = median(Y, 2);
    tY = Y-repmat(M, 1, ns);
else
    tY = Y;
end

nY = sum(abs(tY(:)));

alpha = inf;

% Tuning Parameter
% ma = 1e-6;
% pdratio = 0.1;


count = 0;
% P = zeros(nd, nb);
% X = randn(nb, ns);

[X, ~] = qr(randn(ns, nb), 0);
X = X';
[X, ~, ~] = qr(tY', 0);
X = X(:, 1:nb)';
X = PCA(tY', false);
X = X(:,1:nb)';

P = tY*X';
tY = tY-P*X;

pY = inf;
sY = sum(abs(tY(:)));
while (pY-sY)/nY > th && sY/nY > th
    pY = sY;
    
    [X, R] = qr(X', 0);
    X = X';
    P = P*R';
    
    cnt = 0;
    alpha = max(min(alpha, sY/nd/ns), ma);  %ma, sY/nd/ns : tuning param.
    vt = zeros(nd, ns);
    spY = sY;
    sdY = 0;
    Lc = 1; %
    while (sY-spY)/(sY-sdY) < pdratio
        
        if cnt
            alpha = max(alpha/2, ma);
        end
        
        v = min(max(tY + vt, -alpha), alpha);
        d = v*X';
        dt = d*X;
        vt = v-dt/Lc; %
        
        nx = norm(d(:))^2/2/alpha;
        spY = sum(abs(tY(:)-dt(:))) + nx;
        sdY = tY(:)'*v(:)/alpha - nx;
        
        cnt = cnt + 1;
    end
    
    q = wm_method(tY, dt, nss);
    
    P = P + d*q;
    tY = tY - dt*q;
    
    sY = sum(abs(tY(:)));

    [P, R] = qr(P, 0);
    X = R*X;
    
    cnt = 0;
    alpha = max(min(alpha, sY/nd/ns), ma);
    vt = zeros(nd, ns);
    spY = sY;
    sdY = 0;
    Lc = 1; %
    while (sY-spY)/(sY-sdY) < pdratio
        
        if cnt
            alpha = max(alpha/2, ma);
        end
        
        v = min(max(tY + vt, -alpha), alpha); % -alpha <= tY +vt <= alpha, if tY+vt>= alpha, then tY+vt = alpha and if tY+vt<=-alpha, tY+vt = -alpha
        d = P'*v;
        dt = P*d;
        vt = v-dt/Lc;
        
        nx = norm(d(:))^2/2/alpha;
        spY = sum(abs(tY(:)-dt(:))) + nx;
        sdY = tY(:)'*v(:)/alpha - nx;
        
        cnt = cnt + 1;
        %         disp(['X In ' num2str(cnt) ' - ' num2str(sY/nY) ' : ' num2str(1-spY/sY) ' / ' num2str(1-sdY/sY) ' / ' num2str((sY-spY)/(sY-sdY)) ' / ' num2str(alpha)]);
    end
    
    q = wm_method(tY, dt, nss);
    
    X = X + d*q;
    tY = tY - dt*q;
    
    sY = sum(abs(tY(:)));
    %     disp(['X : ' num2str(sY/nY) ' / ' num2str(q) ' / ' num2str(cnt) ' / ' num2str(alpha)]);
    
    count = count + 1;
    %     disp(['L1-ARG : ' num2str(count) ' : ' num2str((pY-sY)/nY) ' / ' num2str(sY/nY)]);
end
C = count;

end

function q = wm_method(I, a, k)

if numel(I) < 1e5
    [T, ind] = sort(I(:)./a(:));
    p = cumsum(abs(a(ind)));
    q = T(sum(p < 0.5*p(end))+1);
else
    np = 1e5;
    
    if k == 1        % overall data
        q = weighted_median(I(:)./a(:), abs(a(:)));
        
    elseif k == 2    % partial data
        rs = randi(numel(I), 1, np);
        temp = a(rs);
        q = weighted_median(I(rs)./temp, abs(temp));
        
    elseif k == 3    % reflecting weight > median
        p = cumsum(abs(a(:)));
        rp = sort(rand(1, np))*p(end);
        j = 1;
        rs = zeros(1, np);
        for i=1:np
            while p(j) < rp(i)
                j = j+1;
            end
            rs(i) = j;
        end
        q = median(I(rs)./a(rs));
        
    elseif k == 4     % Original weighted median
        [T, ind] = sort(I(:)./a(:));
        p = cumsum(abs(a(ind)));
        q = T(sum(p < 0.5*p(end))+1);
        
    end
end

if isnan(q) || isinf(q)
    q = 0;
end

end

function r = weighted_median(x, p)

if numel(x) < 1e5
    [T, ind] = sort(x);
    q = p(ind);
    l = 0.5*sum(q);
    tt = q(1);
    i = 1;
    while tt < l
        i = i+1;
        tt = tt+q(i);
    end
    r = T(i);
else
    rp = 0;
    r = x;
    q = p/sum(p);
    while numel(r) > 1
        ind = randi(numel(r), 1, ceil(numel(r)/1e3));
        v = weighted_median(r(ind), q(ind));
        
        pind = r < v;
        pp = sum(q(pind));
        if rp+pp < 0.5
            rp = rp+pp;
        else
            r = r(pind);
            q = q(pind);
            continue;
        end
        
        vind = r == v;
        rp = rp+sum(q(vind));
        if rp < 0.5
            nind = ~(pind | vind);
            r = r(nind);
            q = q(nind);
        else
            r = v;
            break;
        end
    end
end

end

function [P, lambda, M] = PCA(X, mz)


if diff(size(X)) < 0
    [Q, R] = qr(X, 0);
    [P, D] = eig(R*R');
    [lambda, ind] = sort(diag(D), 'descend');
    P = Q*P(:, ind);
else
    [P, D] = eig(X*X');
    [lambda, ind] = sort(diag(D), 'descend');
    P = P(:, ind);
end

end
