%% 
clc; clear; close all;

%% 1. 
clc; clear; close all;

nr_data = 100;
hyp = struct('g2', 2E0, 'l2', 1E0, 'w2', 1E-2);
input = linspace(1, 1E1, nr_data)';
output = zeros(nr_data, 1);

% ڡڡ LHS of the leveraged kernel function ڡڡ 
kern_type = 'biased-exp';
switch kern_type
    case 'abs', fun_lev = @(xi, xj)(1 - abs(xi-xj));            % abs <== ORIGINAL + NOT DIFF'LE
    case 'cos', fun_lev = @(xi, xj)(cos((xi-xj)*pi/2));         % cos <== WORKS + DIFF'LE
    case 'exp', fun_lev = @(xi, xj)(2*exp(-(xi-xj).^2));      % exp <== WORKS + DIFF'LE
    case 'biased-exp', fun_lev = @(xi, xj)(2*exp(-(xi-xj).^2)-0.91);  % biased exp <== NOT WORKS + DIFF'LE
    case 'quad', fun_lev = @(xi, xj)(1-0.5*(xi-xj).^2);         % quad <== NOT WORKS + DIFF'LE
    case 'biquad', fun_lev = @(xi, xj)(1-0.1*(xi-xj).^4);       % biquad <== NOT WORKS + DIFF'LE
end
 

% plot 'fun_lev'
fig = figure(1); fi = linspace(-2, 2, 100); fo = fun_lev(fi, 0);
plot(fi, fo); grid on; axis equal; 
drawnow;


% Number of Checks
max_i = 1E2;
fprintf('Checking PSD for %d time. . . \n', max_i);
for i = 1:max_i
    lev = -1+2*rand(nr_data, 1);  % lev varies from -1 to 1
    K = kernel_lev(input, input, lev, lev, hyp, fun_lev);
    eigK = eig(K);
    if min(eigK) < -1E-4 || max(imag(eigK)) ~= 0
        fprintf('[%d/%d] Indefinite Kernel Matrix detected \n', i, max_i);
        fprintf('minimum eigenvalue: %.2e \n', eigK(1));
        break;
    end
end
if i == max_i
    fprintf('It could be PSD kernel (at least for %d times).\n', max_i);
    title(sprintf('%s function is a PSD kernel function', kern_type), 'FontSize', 15);
else
    title(sprintf('%s function is NOT a PSD kernel function', kern_type), 'FontSize', 15);
end

% set(fig,'PaperPositionMode','auto')
% print (fig , '-dpng', ['fig_',kern_type, '.png']) ;

%% 2. kernel_lev_cos_mod  psd ˾ƺ. 
ccc;

nr_data = 100;
hyp = struct('g2', 2E0, 'l2', 1E0, 'w2', 1E-2);
input = linspace(1, 1E1, nr_data)';
output = zeros(nr_data, 1);

% Number of Checks
max_i = 1E2;
fprintf('Checking PSD for %d time. . . \n', max_i);
for i = 1:max_i
    % lev = -1+2*rand(nr_data, 1);  % lev varies from -1 to 1
    nr1 = randi([1 nr_data]);
    nr2 = randi([1 nr_data]);
    if nr1 > nr2
        temp = nr2;
        nr2 = nr1;
        nr1 = temp;
    end
    lev = [ones(1, nr1) zeros(1, nr2-nr1) -ones(1, nr_data-nr2)];
    
    % K = kernel_lev_cos_mod(input, input, lev, lev, hyp);
    K = kernel_lev_cos(input, input, lev, lev, hyp);
    
    eigK = eig(K);
    if min(eigK) < -1E-4 || max(imag(eigK)) ~= 0
        fprintf('[%d/%d] Indefinite Kernel Matrix detected \n', i, max_i);
        fprintf('minimum eigenvalue: %.2e \n', eigK(1));
        break;
    end
end
if i == max_i
    fprintf('It could be PSD kernel (at least for %d times).\n', max_i);
end

figure(1);
imagesc(K);
colorbar;

%% 2 Leveraged Kernel Function ׷. (  leverage case ̿)
ccc;
hyp = struct('g2', 2E0, 'l2', 1E0, 'w2', 1E-2);
res = 300;
X1 = 0; X2 = linspace(-3, 3, res)';
nr_lev = 11;
levs_list = [ones(nr_lev, 1) linspace(-1, 1, nr_lev)'];
fig = figure(1); clf; scrsz = get(0,'ScreenSize');
set(fig,'Position', [0.2*scrsz(3),0.2*scrsz(4),0.4*scrsz(3),0.35*scrsz(4)]);
hold on; set(gca,'FontSize', 14);
colors = jet(size(levs_list, 1));
hs = zeros(size(levs_list, 1), 1);
strs = cell(size(levs_list, 1), 1);
for i = 1:size(levs_list, 1)
    lev1 = levs_list(i, 1);
    lev2 = levs_list(i, 2);
    
    % K_lev = kernel_lev_cos_mod(X1, X2, lev1, lev2*ones(res, 1), hyp);
    K_lev = kernel_lev_cos(X1, X2, lev1, lev2*ones(res, 1), hyp);
    
    hs(i) = plot(X2, K_lev, 'Color', colors(i, :));
    strs{i} = sprintf('lev1: %.1f / lev2: %.1f', lev1, lev2);
end
legend(hs, strs);
grid on;



%% 3 Leveraged Kernel Function ׷. (  X ٸ   Leverage)
ccc;
colormap hsv;
xs = linspace(-3, 3, 5E1); 
levs = linspace(-2, 2, 5E1);
[X1,X2] = meshgrid(xs,levs);
x_list = X1(:);
lev_list = X2(:);
hyp = struct('g2', 2E0, 'l2', 1E0, 'w2', 1E-2);
len = length(x_list);
F = zeros(len, 1);
for i = 1:len
    curr_x = x_list(i);
    curr_lev = lev_list(i);
    % F(i) = kernel_lev_abs(0, curr_x, 0, curr_lev, hyp);
    F(i) = kernel_lev_cos(0, curr_x, 0, curr_lev, hyp);
    % F(i) = kernel_lev_cos_mod(0, curr_x, 0, curr_lev, hyp);
end
F = reshape(F,length(levs),length(xs));

fig = figure(1); 
set(gca,'FontSize', 14);
surfc(xs,levs,F);
xlabel('X difference'); ylabel('Leverage difference'); zlabel('Leveraged kernel function');
view(-50, 30);


%%






