function dK = der_kernel_se(X1, X2, hyp, param)
%
% HYPERPARAMETERS
% hyp.g2: GAIN
%     l2: LENGTH PARAMETER
%     w2: VARIANCE
%

nr_X1 = size(X1, 1);
nr_X2 = size(X2, 1);
L = -sqdist(X1', X2', 1/hyp.l2);
K = hyp.g2*exp(L);
if (nr_X1 == nr_X2) && (nr_X1 > 1)
    K = K + hyp.w2*eye(nr_X1, nr_X1);
end

switch param
    case 'l2'
        M = -L/hyp.l2;
        dK = M.*K;
end

function m = sqdist(p, q, A)
% SQDIST      Squared Euclidean or Mahalanobis distance.
% SQDIST(p,q)   returns m(i,j) = (p(:,i) - q(:,j))'*(p(:,i) - q(:,j)).
% SQDIST(p,q,A) returns m(i,j) = (p(:,i) - q(:,j))'*A*(p(:,i) - q(:,j)).
% Written by Tom Minka
[~, pn] = size(p);
[~, qn] = size(q);
if pn == 0 || qn == 0
    m = zeros(pn,qn);
    return
end
if nargin == 2
    pmag = col_sum(p .* p);
    qmag = col_sum(q .* q);
    m = repmat(qmag, pn, 1) + repmat(pmag', 1, qn) - 2*p'*q;
    %m = ones(pn,1)*qmag + pmag'*ones(1,qn) - 2*p'*q;
else
    Ap = A*p;
    Aq = A*q;
    pmag = col_sum(p .* Ap);
    qmag = col_sum(q .* Aq);
    m = repmat(qmag, pn, 1) + repmat(pmag', 1, qn) - 2*p'*Aq;
end


function Y = col_sum(X)
Y = sum(X, 1);