function lgp_struct = init_lgpr(input, output, kfun, hyp, lev)
%
% Initialize Leveraged Gaussian Process 
%
nr_data = size(input, 1);
if nargin == 4
    % if 'lev' is not set, do stationary GPR
    lev = ones(nr_data, 1);
end

% input, output, lev
lgp_struct.nr_data = nr_data;
lgp_struct.input   = input;
lgp_struct.output  = output;
lgp_struct.lev     = lev;
lgp_struct.hyp     = hyp;

% Set kernel function 
lgp_struct.raw_kernel = kfun;
lgp_struct.kernel_train = @(X1, X2, lev1, lev2) ...
    (kfun(X1, X2, lev1, lev2, lgp_struct.hyp));
lgp_struct.kernel_test = @(X1, X2, lev1, lev2) ...
    (kfun(X1, X2, lev1, lev2, lgp_struct.hyp));

% Precompute kernel matrix
lgp_struct.K = lgp_struct.kernel_train(lgp_struct.input, lgp_struct.input ...
    , lgp_struct.lev, lgp_struct.lev);

lgp_struct.invK = inv(lgp_struct.K);
lgp_struct.invKY = lgp_struct.K \ lgp_struct.output; 

lgp_struct.mean_output = mean(lgp_struct.output);
lgp_struct.invKmeanzeroY = lgp_struct.K \ ...
    (lgp_struct.output - repmat(lgp_struct.mean_output, lgp_struct.nr_data, 1)); 

